/*******************************************************************************
 * Copyright (c) 2000, 2004 QNX Software Systems and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     QNX Software Systems - Initial API and implementation
 *******************************************************************************/
package org.tigcc;
 
import java.io.IOException;

import org.eclipse.cdt.core.AbstractCExtension;
import org.eclipse.cdt.core.CCorePlugin;
import org.eclipse.cdt.core.IBinaryParser;
import org.eclipse.core.runtime.IPath;

/**
 */
public class Ti68kParser extends AbstractCExtension implements IBinaryParser {


	/* (non-Javadoc)
	 * @see org.eclipse.cdt.core.IBinaryParser#getBinary(org.eclipse.core.runtime.IPath)
	 */
	public IBinaryFile getBinary(IPath path) throws IOException {
		return getBinary(null, path);
	}

	/**
	 * 
	 * @param hints
	 * @param path
	 * @return
	 * @throws IOException if path is null
	 */
	public IBinaryFile getBinary(byte[] hints, IPath path) throws IOException {
		if (path == null) {
			throw new IOException(CCorePlugin.getResourceString("Util.exception.nullPath")); //$NON-NLS-1$
		}

		IBinaryFile binary = null;
		Ti68k.Attribute attribute = null;
		if (hints != null && hints.length > 0) {
			attribute = Ti68k.getAttributes(hints);
		}

		//Take a second run at it if the data array failed. 			
		if(attribute == null) {
			attribute = Ti68k.getAttributes(path.toOSString());
		}

		if (attribute != null) {
			switch (attribute.getType()) {
				case Ti68k.Attribute.TI68K_TYPE_EXE :
					binary = createBinaryExecutable(path);
					break;

				case Ti68k.Attribute.TI68K_TYPE_SHLIB :
					binary = createBinaryShared(path);
					break;

				case Ti68k.Attribute.TI68K_TYPE_OBJ :
					binary = createBinaryObject(path);
					break;

				case Ti68k.Attribute.TI68K_TYPE_CORE :
					binary = createBinaryCore(path);
					break;
			}
		}
		return binary;
	}

	/**
	 * @see org.eclipse.cdt.core.model.IBinaryParser#getFormat()
	 */
	public String getFormat() {
		return "ELF"; //$NON-NLS-1$
	}

	/* (non-Javadoc)
	 * @see org.eclipse.cdt.core.IBinaryParser#isBinary(byte[], org.eclipse.core.runtime.IPath)
	 */
	public boolean isBinary(byte[] array, IPath path) {
		return Ti68k.isTi68kHeader(array);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.cdt.core.IBinaryParser#getBufferSize()
	 */
	public int getHintBufferSize() {
		return 128;
	}

	/**
	 * @param path
	 * @return
	 */
	protected IBinaryArchive createBinaryArchive(IPath path) {
		return new Ti68kBinaryArchive(this, path);
	}

	protected IBinaryObject createBinaryObject(IPath path) {
		return new Ti68kBinaryObject(this, path, IBinaryFile.OBJECT);
	}

	protected IBinaryExecutable createBinaryExecutable(IPath path) {
		return new Ti68kBinaryExecutable(this, path);
	}

	protected IBinaryShared createBinaryShared(IPath path) {
		return new Ti68kBinaryShared(this, path);
	}

	protected IBinaryObject createBinaryCore(IPath path) {
		return new Ti68kBinaryObject(this, path, IBinaryFile.CORE);
	}
}
